/* C++ */

#ifndef SC_INT_BASE_H
#define SC_INT_BASE_H

#include <sc_common.h>

namespace sc_dt
{
    class sc_int_bitref_r
    {
        public:
        sc_int_bitref_r(int64& data, int_type index):
            data(data), index(index){}

        operator bool() const;

        private:

        // disabled
        sc_int_bitref_r& operator = (const sc_int_bitref_r&);

        int64& data;
        int_type index;
    };

    class sc_int_bitref
    {
        public:
        sc_int_bitref(int64& data, int_type index):
        data(data), index(index){}

        operator bool() const;

        sc_int_bitref& operator = (bool);

        private:
        int64& data;
        int_type index;
    };

    class sc_int_subref_r
    {
        public:
        sc_int_subref_r(const int64& data, int_type left, int_type right):
            data(data), right(right), left(left){}

        operator int_type () const;

        private:

        // disabled
        sc_int_subref_r& operator = (const sc_int_subref_r&);

        const int64& data;
        int_type right;
        int_type left;
    };

    class sc_int_subref
    {
        public:
        sc_int_subref(int64& data, int_type left, int_type right):
            data(data), left(left), right(right){};

        operator int_type() const
        { return (int_type)(*((const sc_int_subref_r *)this)); }

        sc_int_subref& operator = (int_type);

        private:
        int64& data;
        int_type right;
        int_type left;
    };


    class sc_int_base
    {
        friend class sc_int_subref;
        friend class sc_int_subref_r;

        public:

        sc_int_base& operator=(const sc_int_base& a);
        sc_int_base& operator=(int_type a);
        sc_int_base& operator=(uint_type a);

        bool operator ==(const sc_int_base& a) const;
        bool operator !=(const sc_int_base& a) const;
        bool operator < (const sc_int_base& a) const;
        bool operator > (const sc_int_base& a) const;
        bool operator >= (const sc_int_base& a)const;
        bool operator <= (const sc_int_base& a)const;


        sc_int_base& operator++();
        sc_int_base operator++(int_type);
        sc_int_base& operator--();
        sc_int_base  operator--(int_type);


        sc_int_base& operator+=(const sc_int_base& a);
        sc_int_base& operator-=(sc_int_base a);

        sc_int_base& operator &=(const sc_int_base& a);
        sc_int_base& operator |=(const sc_int_base& a);
        sc_int_base& operator ^=(const sc_int_base& a);

        sc_int_base& operator >>=(uint_type a);
        sc_int_base& operator <<=(uint_type a);

        operator int_type() const;

        sc_int_subref range(int_type left, int_type right)
        {
            __CPROVER_assert(widht > left && left >= right && right >=0, "bad range");
            return sc_int_subref(data, left, right);
        }

        sc_int_subref_r range(int_type left,int_type right) const
        {
            __CPROVER_assert(widht > left && left >= right && right >=0, "bad range");
            return sc_int_subref_r(data, left, right);
        }

        // syntaxic sugar
        sc_int_subref operator()(int_type left,int_type right)
        { return range(left,right);}

        sc_int_subref_r operator()(int_type left, int_type right) const
        { return range(left,right);}


        int_type length() const
        { return width; }

        protected:
        sc_int_base(int_type width): width(width){}
        sc_int_base(int_type width, int64 data):
            width(width), data(data){}

        int64 data;
        int_type width;
    };
}
#endif
